function [ indexset_cell, coeff_cell ] = build_signals_symhc( d, N, M, num_signals, filename )
%BUILD_SIGNALS_SYMHC Creates sparse random multi-dimensional test signals with
%frequencies from hyperbolic cross and Fourier coefficients on the unit circle.
%
%   Input parameters:
%      d - dimension
%      N - parameter for index set \{k\in Z^d: \prod_{t=1}^d max(1,|k|) <= N \}
%          of possible frequencies
%      M - signal sparsity
%      num_signals - number of test signals to generate
%      filename - output file name for test signals
%
%   Output parameters:
%      indexset_cell - cell array (length num_signals) containing array 
%                      siz
%                      of length M of frequencies
%      coeff_cell - cell array (length num_signals) containing column vectors
%                   of length M of Fourier coefficients
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if exist(filename,'file') ~= 0
  error('File %s already exists', filename);
end

indexset_cell = cell(num_signals,1);
coeff_cell = cell(num_signals,1);

freq = double(coord_symhc(N,d,1));
lfreq = size(freq,1);

for ir=1:num_signals
  ind = [];
  while length(ind) < M
    ind = [ind; round(lfreq * rand(2*M,1) + 0.5)];
    ind = unique(ind, 'stable');
  end
  ind = ind(1:M);
  indexset_in = freq(ind,:);
  indexset_cell{ir} = indexset_in;

  fc_min = 1;
  coeff_in = exp(2*pi*1i*rand(M,1));
  coeff_cell{ir} = coeff_in;
end

save(filename, 'indexset_cell', 'coeff_cell', 'num_signals', 'd', 'N', 'fc_min');

end
