function [h]=coord_symhc(N,maxd,gamma,opt)
%
% h=coord_symhc(N,d,gamma)
%
%   computes hyperbolic cross index sets containing all h\in\mathbb{Z}^d
%   with
%   
%   \prod_{s=1}^d\max(1,|h_s|/gamma_s)\le N
%
%   N:      refinement
%   d:      dimension 
%   gamma:  descending ordered vector of nonnegative weights of length d
%           1\ge gamma_1\ge \ldots \gamma_d>0
%           sometimes it's better to use weights+eps to avoid round down
%           errors
%   opt.parity
%           even: only even indices
%           odd:  only odd indices (0 is even and odd)
%           none: default
%
% Copyright (c) 2014 Lutz Kaemmerer

    if(nargin==3)
        opt=0;
    end

    if prod(size(gamma))==1
        gamma=ones(1,maxd)*gamma;
    end

    a1=(0:N*gamma(1))';
    a1=double2int(a1,gamma(1)*N);
    weight=max(1,double(a1)./gamma(1));
    if(isfield(opt,'parity'))
        if(strcmp(opt.parity,'even')==1)
            ind=find(mod(a1,2)==0);
            weight=weight(ind);
            a1=a1(ind);
        elseif(strcmp(opt.parity,'odd')==1)
            ind=find(mod(a1,2)==1);
            ind0=find(a1==0);
            weight=weight([ind0;ind]);
            a1=a1([ind0;ind]);
        end
    end
    
    
    
for d=2:maxd
%     if(mod(d,20)==0)
%         fprintf('d=%d, ',d);
%     end
    a2=[];
    a2=double2int(a2,gamma(1)*N);
    weight2=[];

    for l=1:size(a1,1)
        tmp=double2int(0:N*gamma(d)/weight(l),gamma(1)*N);
        a2=[a2;double2int([(ones(1,floor(N*gamma(d)/weight(l))+1)'*double(a1(l,:))),tmp'], gamma(1)*N)];
        tmp=max(1,double(tmp)/gamma(d));
        weight2=[weight2;weight(l)*tmp'];
    end
    a1=a2;
    weight=weight2;
    if(isfield(opt,'parity'))
        if(strcmp(opt.parity,'even')==1)
            ind=find(mod(a1(:,d),2)==0);
            weight=weight(ind);
            a1=a1(ind,:);
        elseif(strcmp(opt.parity,'odd')==1)
            ind=find(mod(a1(:,d),2)==1);
            ind0=find(a1(:,d)==0);
            weight=weight([ind0;ind]);
            a1=a1([ind0;ind],:);
        end
    end
end

%disp('call sym_set ...');
h=sym_set(a1);
%h=old_sym_set(a1);
end

function b=sym_set(a)

    tmp=[a];
    tmp1=tmp(tmp(:,1)~=0,:);
    tmp=[tmp;-tmp1(:,1),tmp1(:,2:size(tmp1,2))];
    tmp=unique(tmp, 'rows');
    for j=2:size(a,2)-1
        tmp1=tmp(tmp(:,j)~=0,:);
        tmp=[tmp;tmp1(:,1:j-1),-tmp1(:,j),tmp1(:,j+1:size(a,2))];
        %tmp=unique(tmp, 'rows');
    end
    tmp1=tmp(tmp(:,size(a,2))~=0,:);
    tmp=[tmp;tmp1(:,1:size(a,2)-1),-tmp1(:,size(a,2))];
    %tmp=unique(tmp, 'rows');
    if(size(a,2)==1)
        tmp=unique(tmp,'rows');
    end
    b=tmp;
end

function b=old_sym_set(a)

    tmp=[a];
    tmp=[tmp;-tmp(:,1),tmp(:,2:size(tmp,2))];
    tmp=unique(tmp, 'rows');
    for j=2:size(a,2)-1
        tmp=[tmp;tmp(:,1:j-1),-tmp(:,j),tmp(:,j+1:size(a,2))];
        tmp=unique(tmp, 'rows');
    end
    tmp=[tmp;tmp(:,1:size(a,2)-1),-tmp(:,size(a,2))];
    tmp=unique(tmp, 'rows');
    b=tmp;
end

function [a]=double2int(a, maxint)

    if maxint<=2^7-1
        a=int8(a);
    else
        a=int16(a);
    end

end