% Runs some of the tests of Example 5.7 from
% Potts, D., Tasche, M., Volkmer, T.
% Efficient spectral estimation by MUSIC and ESPRIT with application to sparse FFT.
% Front. Appl. Math. Stat. 2, 2016.

SNR_values = [Inf, 10^10];
d = 6;
symhc_N = 16;
lattice_S = 1105193;
lattice_z = [1,33,579,3628,21944,169230];
sparsity_M = 256;
num_signals = 100;
algorithm_name = 'esprit';
K_array = 12;
K_SVD_threshold_array = 12;
P_array = 64;
niter_max = 10;
epsilon_svd_list = 10.^(-3:-1:-8);

signals_filename = sprintf('signals/signals_symhc_N%d_d%d_M%d_r%d.mat', symhc_N, d, sparsity_M, num_signals);
if exist(signals_filename,'file') == 0
  build_signals_symhc( d, symhc_N, sparsity_M, num_signals, signals_filename );
end
for SNR = SNR_values
  testr1l_symhc_run_testsets( SNR, algorithm_name, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list );
end
for SNR = SNR_values
  print_symhc_testr1l( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, algorithm_name, K_array, K_SVD_threshold_array, P_array, niter_max );
end


% % Run (almost) all tests from Example 5.7, may take a very very long time.
% d = 6;
% symhc_N = 16;
% lattice_S = 1105193;
% lattice_z = [1,33,579,3628,21944,169230];
% sparsity_M = 256;
% num_signals = 100;
% algorithm_name = 'esprit';
% niter_max = 10;
% epsilon_svd_list = 10.^(-3:-1:-8);
% 
% signals_filename = sprintf('signals/signals_symhc_N%d_d%d_M%d_r%d.mat', symhc_N, d, sparsity_M, num_signals);
% if exist(signals_filename,'file') == 0
%   build_signals_symhc( d, symhc_N, sparsity_M, num_signals, signals_filename );
% end
% 
% % noiseless case
% SNR = Inf; P_array = [8 16 32 64 128];
% K_array = [8 10 12]; K_SVD_threshold_array = [8 10 12];
% testr1l_symhc_run_testsets( SNR, algorithm_name, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list );
% 
% % noisy case
% SNR = 10^10; P_array = [32 64 128];
% K_array = [12 12 24]; K_SVD_threshold_array = [6 12 12];
% testr1l_symhc_run_testsets( SNR, algorithm_name, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list );
% 
% SNR = 10^8; P_array = [32 64 128];
% K_array = [12 12 24]; K_SVD_threshold_array = [6 12 12];
% testr1l_symhc_run_testsets( SNR, algorithm_name, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list );
% 
% SNR = 10^6; P_array = [32 64 128 256 512];
% K_array = [12 12 24]; K_SVD_threshold_array = [6 12 12];
% testr1l_symhc_run_testsets( SNR, algorithm_name, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list );
% 
% % print results
% SNR = Inf; P_array = [8 16 32 64 128];
% K_array = [8 10 12]; K_SVD_threshold_array = [8 10 12];
% print_symhc_testr1l( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, algorithm_name, K_array, K_SVD_threshold_array, P_array, niter_max );
% 
% SNR = 10^10; P_array = [32 64 128];
% K_array = [12 12 24]; K_SVD_threshold_array = [6 12 12];
% print_symhc_testr1l( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, algorithm_name, K_array, K_SVD_threshold_array, P_array, niter_max );
% 
% SNR = 10^8; P_array = [32 64 128];
% K_array = [12 12 24]; K_SVD_threshold_array = [6 12 12];
% print_symhc_testr1l( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, algorithm_name, K_array, K_SVD_threshold_array, P_array, niter_max );
% 
% SNR = 10^6; P_array = [32 64 128 256 512];
% K_array = [12 12 24]; K_SVD_threshold_array = [6 12 12];
% print_symhc_testr1l( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, algorithm_name, K_array, K_SVD_threshold_array, P_array, niter_max );
