function f=lfft(h,z,M,f_hat,tflag)
%LFFT Computes rank-1 lattice FFT.
%
% f=lfft(h,z,M,fhat,tflag)
%
%   h:      frequency index set
%   z:      generating integer vector
%   M:      lattice size
%   fhat:   Fourier coefficients f_hat(j)=f_hat_h(j,:) with tflag='notransp'
%           Function values f_hat(j)=f(mod(j*z/M,1)) with tflag='transp'
%   tflag:  'notransp' -> Evaluation of 
%
%               f(j+1)=sum_{k\in h}f_hat(k)*e^(2*pi*i*j*k'z/M), j=0:M-1
%
%           'transp' -> Evaluation of transposed
%
%               f(l)=sum_{j=0}^M f_hat(j)*e^(2*pi*i*j*k(l)'z/M),
%                                           k(l)=h(l,:), l=1:size(h,1)
%
%   Note: 
%
%       lfft(h,z,M,lfft(h,z,M,fhat,'notransp'),'transp')=M*fhat
%           
%       in the case that z and M generates a reconstruction lattice
%
% Copyright (c) 2014 Lutz Kaemmerer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

    if(~isfloat(h))
        h=double(h);
    end
    switch(tflag)
        case 'notransp'
            f=lhcfft_notransp(h,z,M,f_hat);
        case 'transp'
            f=lhcfft_transp(h,z,M,f_hat);
    end

end

function f=lhcfft_notransp(H,z,M,f_hat)

    k=mod(H*z',M)+1;
    fhat1=accumarray(k,f_hat,[M,1],@sum);
    
    f=M*ifft(fhat1);
end

function fhat=lhcfft_transp(H,z,M,f)
%    ghat=M*ifft(conj(f));
    ghat=fft(f);
    k=mod(H*z',M)+1;
    fhat=ghat(k);
%    fhat=zeros(1,length(k));
%    for j=1:length(k)
%        fhat(j)=ghat(k(j));
%    end
end
